/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gems;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import org.greenstone.gatherer.util.Utility;


/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class AttributeTableModel 
    extends AbstractTableModel {
    private Attribute last_attribute = null;
    private int last_index = -1;
    private int longest[] = null;
    private int column_zero_width = 0;
    private JScrollPane scroll_pane = null;
    private JTable table = null;
    private String default_value = null;
    private String names[] = null;
    private TreeSet attributes = null;
    static final private int table_width = 550;
    public AttributeTableModel(TreeSet attributes, String one, String two, String default_value) {
	super();
	this.attributes = attributes;
	this.default_value = default_value;
	this.longest = new int[2];
	this.names = new String[2];
	names[0] = one;
	names[1] = two;
    }

    public AttributeTableModel(TreeSet attributes, String one, String two, String three, String default_value) {
	super();
	this.attributes = attributes;
	this.default_value = default_value;
	this.longest = new int[3];
	this.names = new String[3];
	names[0] = one;
	names[1] = two;
	names[2] = three;
    }
    public int add(Attribute attribute) {
	attributes.add(attribute);
	// Determine the row index
	boolean found = false;
	int index = 0;
	Iterator iterator = attributes.iterator();
	for(int i = 0; !found && iterator.hasNext(); i++) {
	    if(attribute == iterator.next()) {
		found = true;
		index = i;
	    }
	}
	// Fire event
	fireTableRowsInserted(index, index);
	return index;
    }

    /**
     * Checks to see if the table contains a given value in a given column
     *
     * @param value the <strong>String</strong> to check for
     * @param column the column number to check in, as an <strong>int</strong>
     * @return boolean - true if the table contains the value.
     */
    public boolean contains(String value, int column) {
	boolean result = false;
	Iterator iterator = attributes.iterator();
	while(iterator.hasNext() && !result) {
	    Attribute attribute = (Attribute) iterator.next();
	    if(value.equals(attribute.get(column))) {
		result = true;
	    }
	}  
	return result;
    }

    /**
     * Checks to see if the table contains a given value in a given language
     *
     * @param value the <strong>String</strong> to check for
     * @param language_code the language code to check for, as a <strong>String</strong>
     * @return boolean - true if the table contains the value.
     */
    public boolean contains(String value, String language_code) {
	Iterator iterator = attributes.iterator();
	while(iterator.hasNext()) {
	    Attribute attribute = (Attribute) iterator.next();
	    String attributeLanguage = attribute.language;
	    if(attributeLanguage == null) {
		attributeLanguage = "";
	    }	    
	    if(language_code == null) {
		language_code = "";
	    }
	  
	    if(value.equals(attribute.name) && language_code.equalsIgnoreCase(attribute.language)) {
		return true;
	    }
	}
	return false;
    }

    public Attribute getAttribute(int row) {
	if(row != last_index) {
	    last_index = row;
	    last_attribute = null;
				// Retrieve the value from the set using its iterator
	    Iterator iterator = attributes.iterator();
	    for(int i = 0; i < row && iterator.hasNext(); i++) {
		iterator.next();
	    }
	    if(iterator.hasNext()) {
		last_attribute = (Attribute) iterator.next();
	    }
	    iterator = null;
	}
	return last_attribute;
    }

    public int getColumnCount() {
	return names.length;
    }

    public String getColumnName(int col) {
	return names[col];
    }

    public int getRowCount() {
	int count = 0;
	if(attributes != null) {
	    count = attributes.size();
	}
	return count;
    }

    public Object getValueAt(int row, int col) {
	String value = null;
	Attribute attribute = getAttribute(row);
	if(attribute != null) {
	    value = attribute.get(col);
	}
	if(value == null) {
	    value = default_value;
	}
	// Update table column width if necessary
	if(value.length() > longest[col]) {
	    Component component = table.getCellRenderer(row, col).getTableCellRendererComponent(table, value, false, false, 0, 0);
	    int width = component.getPreferredSize().width + 10;
	    TableColumn column = table.getColumnModel().getColumn(col);
	    if(width > column.getPreferredWidth()) {
		// Calculate default column width
		int default_width = 0;
		int total_width = scroll_pane.getSize().width;
		if(names.length == 2) {
		    if(col == 0) {
			default_width = total_width / 3;
		    }
		    else {
			default_width = 2 * (total_width / 3);
		    }
		}
		else {
		    switch(col) {
		    case 0:
			default_width = (2 * total_width) / 9;
			break;
		    case 1:
			default_width = total_width / 9;
			break;
		    default:
			default_width = 2 * (total_width / 3);
			break;
		    }
		}
		if(width > default_width) {
		    column.setPreferredWidth(width);
		}
		else {
		    column.setPreferredWidth(default_width);
		}
	    }
	    longest[col] = value.length();
	}
	return Utility.stripNL(value);
    }
    public void removeRow(final int row) {
	///ystem.err.println("Remove row " + row);
	Attribute attribute = getAttribute(row);
	if(attribute != null) {
	    attributes.remove(attribute);
				// Arg... this caused me a headache for hours.
	    last_index = -1;
	    fireTableRowsDeleted(row, row);
	}
    }
    public void setScrollPane(JScrollPane scroll_pane) {
	this.scroll_pane = scroll_pane;
    }
    public void setTable(JTable table) {
	this.table = table;
	this.longest = new int[getColumnCount()];
    }
}
